@@ -16,7 +16,7 @@ build_time_vars = {'ABIFLAGS': '',
  'BLDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'aarch64-unknown-openbsd7.0',
+ 'BUILD_GNU_TYPE': 'x86_64-unknown-openbsd7.1',
  'BYTESTR_DEPS': '\\',
  'CC': 'cc -pthread',
  'CCSHARED': '-fPIC',
@@ -36,11 +36,9 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFIGURE_CPPFLAGS': '-I/usr/local/include',
  'CONFIGURE_LDFLAGS': '-L/usr/local/lib/',
  'CONFIGURE_LDFLAGS_NODIST': '',
- 'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-loadable-sqlite-extensions' "
-                "'--enable-shared' "
-                "'--srcdir=/usr/obj/ports/Python-3.9.9/Python-3.9.9' "
-                "'--with-fpectl' '--with-threads' '--enable-ipv6' "
-                "'--with-system-expat' '--with-system-ffi' "
+ 'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-shared' '--with-fpectl' "
+                "'--with-threads' '--enable-ipv6' '--with-system-expat' "
+                "'--with-system-ffi' '--enable-loadable-sqlite-extensions' "
                 "'--prefix=/usr/local' '--sysconfdir=/etc' "
                 "'--mandir=/usr/local/man' '--infodir=/usr/local/info' "
                 "'--localstatedir=/var' '--disable-silent-rules' "
@@ -49,8 +47,8 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python3.9',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/usr/obj/ports/Python-3.9.9/Python-3.9.9/coverage.info',
- 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.9.9/Python-3.9.9/lcov-report',
+ 'COVERAGE_INFO': '/usr/obj/ports/Python-3.9.12/Python-3.9.12/coverage.info',
+ 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.9.12/Python-3.9.12/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -I./Include -I/usr/local/include',
@@ -210,8 +208,8 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_GAI_STRERROR': 1,
  'HAVE_GAMMA': 1,
  'HAVE_GCC_ASM_FOR_MC68881': 0,
- 'HAVE_GCC_ASM_FOR_X64': 0,
- 'HAVE_GCC_ASM_FOR_X87': 0,
+ 'HAVE_GCC_ASM_FOR_X64': 1,
+ 'HAVE_GCC_ASM_FOR_X87': 1,
  'HAVE_GCC_UINT128_T': 1,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETC_UNLOCKED': 1,
@@ -279,6 +277,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_LIBUUID': 0,
  'HAVE_LINK': 1,
  'HAVE_LINKAT': 1,
+ 'HAVE_LINUX_AUXVEC_H': 0,
  'HAVE_LINUX_CAN_BCM_H': 0,
  'HAVE_LINUX_CAN_H': 0,
  'HAVE_LINUX_CAN_J1939_H': 0,
@@ -441,6 +440,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_SYSCONF': 1,
  'HAVE_SYSEXITS_H': 1,
  'HAVE_SYS_AUDIOIO_H': 1,
+ 'HAVE_SYS_AUXV_H': 0,
  'HAVE_SYS_BSDTTY_H': 0,
  'HAVE_SYS_DEVPOLL_H': 0,
  'HAVE_SYS_DIR_H': 0,
@@ -518,16 +518,16 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_X509_VERIFY_PARAM_SET1_HOST': 1,
  'HAVE_ZLIB_COPY': 1,
  'HAVE__GETPTY': 0,
- 'HOST_GNU_TYPE': 'aarch64-unknown-openbsd7.0',
+ 'HOST_GNU_TYPE': 'x86_64-unknown-openbsd7.1',
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include '
                    '/usr/local/include/python3.9 /usr/local/include/python3.9',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python3.9',
- 'INSTALL': '/usr/obj/ports/Python-3.9.9/bin/install -c',
- 'INSTALL_DATA': '/usr/obj/ports/Python-3.9.9/bin/install -c -m 644',
- 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.9.9/bin/install -c  -m 755',
- 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.9.9/bin/install -c -m 755',
- 'INSTALL_SHARED': '/usr/obj/ports/Python-3.9.9/bin/install -c -m 755',
+ 'INSTALL': '/usr/obj/ports/Python-3.9.12/bin/install -c',
+ 'INSTALL_DATA': '/usr/obj/ports/Python-3.9.12/bin/install -c -m 644',
+ 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.9.12/bin/install -c  -m 755',
+ 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.9.12/bin/install -c -m 755',
+ 'INSTALL_SHARED': '/usr/obj/ports/Python-3.9.12/bin/install -c -m 755',
  'INSTSONAME': 'libpython3.9.so.0.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
@@ -660,12 +660,12 @@ build_time_vars = {'ABIFLAGS': '',
                    '-I/usr/local/include -I./Include/internal -I. -I./Include '
                    '-I/usr/local/include -fPIC -DPy_BUILD_CORE',
  'PY_CORE_LDFLAGS': '-L/usr/local/lib/ '
-                    '-L/usr/obj/ports/Python-3.9.9/Python-3.9.9 '
+                    '-L/usr/obj/ports/Python-3.9.12/Python-3.9.12 '
                     '-L/usr/local/lib/',
  'PY_CPPFLAGS': '-I. -I./Include -I/usr/local/include',
  'PY_FORMAT_SIZE_T': '"z"',
  'PY_LDFLAGS': '-L/usr/local/lib/',
- 'PY_LDFLAGS_NODIST': '-L/usr/obj/ports/Python-3.9.9/Python-3.9.9 '
+ 'PY_LDFLAGS_NODIST': '-L/usr/obj/ports/Python-3.9.12/Python-3.9.12 '
                       '-L/usr/local/lib/',
  'PY_SSL_DEFAULT_CIPHERS': 1,
  'PY_SSL_DEFAULT_CIPHER_STRING': 0,
@@ -686,7 +686,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.9.9/Python-3.9.9',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.9.12/Python-3.9.12',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SHELL': '/bin/sh',
@@ -727,10 +727,10 @@ build_time_vars = {'ABIFLAGS': '',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.9.9/Python-3.9.9 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.9.12/Python-3.9.12 '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.9.9/Python-3.9.9 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.9.12/Python-3.9.12 '
                './python ./Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 1200,
  'TIMEMODULE_LIB': 0,
@@ -753,8 +753,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-3.9.9/Python-3.9.9',
- 'abs_srcdir': '/usr/obj/ports/Python-3.9.9/Python-3.9.9',
+ 'abs_builddir': '/usr/obj/ports/Python-3.9.12/Python-3.9.12',
+ 'abs_srcdir': '/usr/obj/ports/Python-3.9.12/Python-3.9.12',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
  'prefix': '/usr/local',
